# Copyright (c) 2007, 2008  Dwayne C. Litzenberger <dlitz@dlitz.net>
# Copyright (c) 2011 Outpost Embedded, LLC
# 
# This file is part of cloudfiles-sftpd.
#
# cloudfiles-sftpd is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# cloudfiles-sftpd is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, logging
from argparse import ArgumentParser

from sclapp2.signals import SafeSignalContext, SignalInterrupt

from cloudfiles_sftpd import __version__
from cloudfiles_sftpd.server import SFTPServer
from cloudfiles_sftpd.conf import CloudFilesSftpdConfigParser


logger = logging.getLogger(__name__) 


def get_user_log_level(name):
    if name == 'debug':
        return logging.DEBUG
    elif name == 'info':
        return logging.INFO
    elif name == 'warning':
        return logging.WARNING
    elif name == 'error':
        return logging.ERROR
    else:
        raise ValueError(name)


def configure_logging(path, name):
    logger = logging.getLogger()
    logger.setLevel(get_user_log_level(name))

    if path == '-':
        handler = logging.StreamHandler()
    else:
        handler = logging.FileHandler(path)

    formatter = logging.Formatter('%(asctime)s %(levelname)s %(name)s %(message)s')
    handler.setFormatter(formatter)

    logger.addHandler(handler)


def load_extensions(extensions):
    logger = logging.getLogger(__name__)
    for name in extensions:
        try:
            __import__(name)
        except:
            logger.exception('failed to load extension %s', name)


def main(argv):
    parser = ArgumentParser()

    parser.add_argument('--config', required = True)
    
    opts = parser.parse_args(argv[1:])

    config = CloudFilesSftpdConfigParser()
    config.read(opts.config)

    configure_logging(config.get_log(), config.get_log_level())
    logger.info('cloudfiles-sftpd version %s', __version__)

    load_extensions(config.get_extensions())

    server = SFTPServer.ForConfig(config)
    server.daemon_threads = False
    try:
        with SafeSignalContext(
          SIGHUP = SignalInterrupt,
          SIGINT = SignalInterrupt,
          SIGTERM = SignalInterrupt,
        ):
            try:
                server.serve_forever()
            except:
                server.close_transports()
                raise
    except KeyboardInterrupt:
        pass
