# Copyright (c) 2007, 2008  Dwayne C. Litzenberger <dlitz@dlitz.net>
# Copyright (c) 2011 Outpost Embedded, LLC
# 
# This file is part of cloudfiles-sftpd.
#
# cloudfiles-sftpd is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# cloudfiles-sftpd is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging

from paramiko import (
  ServerInterface,
  AUTH_SUCCESSFUL,
  AUTH_FAILED,
  OPEN_SUCCEEDED,
  OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED,
)

from cloudfiles_sftpd.auth import AuthMethod
from cloudfiles_sftpd.util import logged


logger = logging.getLogger(__name__)


class CloudFilesServerInterface(ServerInterface):
    auth_manager = None
    active_auth_method = None

    def __init__(self, auth_manager):
        super(CloudFilesServerInterface, self).__init__()

        self.auth_manager = auth_manager

    @logged(logger)
    def get_allowed_auths(self, username):
        return self.auth_manager.get_allowed_auths(username)

    def _handle_check_result(self, auth_method, result):
        if result == AUTH_SUCCESSFUL:
            self.active_auth_method = auth_method
        else:
            self.active_auth_method = None
        return result

    @logged(logger)
    def check_auth_none(self, username):
        auth_method, result = self.auth_manager.check_auth_none(username)
        return self._handle_check_result(auth_method, result)

    @logged(logger)
    def check_auth_password(self, username, password):
        auth_method, result = self.auth_manager.check_auth_password(
          username,
          password,
        )
        return self._handle_check_result(auth_method, result)

    @logged(logger)
    def check_auth_publickey(self, username, key):
        auth_method, result = self.auth_manager.check_auth_publickey(
          username,
          key,
        )
        return self._handle_check_result(auth_method, result)

    @logged(logger)
    def check_channel_request(self, kind, chanid):
        if kind == 'session':
            return OPEN_SUCCEEDED
        return OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED
