# Copyright (c) 2011 Outpost Embedded, LLC
# 
# This file is part of cloudfiles-sftpd.
#
# cloudfiles-sftpd is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# cloudfiles-sftpd is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from functools import wraps
from itertools import chain


try:
    BaseException
except NameError:
    BaseException = Exception


def logged(logger):
    def decorator(fn):
        @wraps(fn)
        def wrapped(*args, **kwargs):
            s_args = ', '.join(chain(
              (repr(arg) for arg in args),
              ('%s=%r' % (k, v) for k, v in kwargs.items()),
            ))
            logger.debug('%s(%s)', fn.__name__, s_args)
            try:
                retval = fn(*args, **kwargs)
            except BaseException, e:
                logger.debug('%s(%s): %r', fn.__name__, s_args, e)
                raise
            logger.debug('%s(%s) -> %r', fn.__name__, s_args, retval)
            return retval
        return wrapped
    return decorator
